using System;
using System.Collections.Generic;
using System.Text;

namespace BinaryTreeTest
{
    public class BinaryTree<T>
    {
        private TreeItem<T> _head;
        public TreeItem<T> Head
        {
            get { return _head; }
            set { _head = value; }
        }

        private T ItemToAdd;
        public void Add(T Item)
        {
            ItemToAdd = Item;
            if (Head == null)
                Head = AddNode(Head);
            else
                AddNode(Head);
        }

        public void Add(params T[] Items)
        {
            foreach (T item in Items)
            {
                Add(item);
            }
        }

        private TreeItem<T> AddNode(TreeItem<T> currentNode)
        {
            if (currentNode == null)
            {
                TreeItem<T> ti = new TreeItem<T>();
                ti.Value = ItemToAdd;
                return ti;
            }
            else if (currentNode.Value is IComparable)
            {
                IComparable currValue = currentNode.Value as IComparable;
                IComparable nodeValue = ItemToAdd as IComparable;
                if (currValue.CompareTo(nodeValue) > 0)
                {
                    currentNode.Left = AddNode(currentNode.Left);
                }
                else
                {
                    currentNode.Right = AddNode(currentNode.Right);
                }
            }
            else if (currentNode.Value is IComparable<T>)
            {
                IComparable<T> currValue = currentNode.Value as IComparable<T>;
                if (currValue.CompareTo(ItemToAdd) > 0)
                {
                    currentNode.Left = AddNode(currentNode.Left);
                }
                else
                {
                    currentNode.Right = AddNode(currentNode.Right);
                }
            }
            return currentNode;
        }

        public IEnumerable<T> InOrder
        {
            get
            {
                return ScanInOrder(Head);
            }
        }

        private IEnumerable<T> ScanInOrder(TreeItem<T> node)
        {
            if (node != null)
            {
                foreach (T value in ScanInOrder(node.Left))
                    yield return value;
                yield return node.Value;
                foreach (T value in ScanInOrder(node.Right))
                    yield return value;
            }
        }
    }
}
